﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using TSLab.Script.Handlers;
using TSLab.Script.Helpers;

namespace RusAlgo.Handlers.Public.Adaptive
{
    [HandlerCategory("RusAlgo")]
    [HandlerName("Adaptive EMA")]
    [InputInfo(0, "Данные")]
    [InputInfo(1, "Период")]
    public class AEma : ITwoSourcesHandler, IDoubleInput0, IDoubleInput1, IDoubleReturns, IStreamHandler, IValuesHandlerWithNumber, IContextUses
    {
        private double[] _dataCache;

        public IContext Context { set; private get; }

        public IList<double> Execute(IList<double> price, IList<double> period)
        {
            var values = new double[Context.BarsCount];

            for (var i = 0; i < Context.BarsCount; i++)
            {
                var currPeriod = Math.Max(Convert.ToInt32(period[i]), 1);
                var value = Context.GetData("AdaptiveEmaRusAlgo",
                                            new[]
                                                {
                                                    currPeriod.ToString(CultureInfo.InvariantCulture),
                                                    price.GetHashCode().ToString(CultureInfo.InvariantCulture)
                                                },
                                            () => Series.EMA(price, currPeriod));
                values[i] = value[i];
            }

            return values;
        }

        public double Execute(double price, double period, int barNum)
        {
            if (_dataCache == null)
                _dataCache = new double[Context.BarsCount];

            _dataCache[barNum] = price;

            var currPeriod = Math.Max(Convert.ToInt32(period), 1);
            // собираем все данные начиная с текущей свечки и назад на величину периода
            // всякие индюки на базе EMA имеют трабл с бесконечной памятью. Даем больше данных чтобы сгладить эту шнягу.
            var currData = _dataCache.CopyCountTo(barNum, currPeriod*5, false);

            // проверочка на случай кривизны
            if (currData.Last() != price)
                throw new Exception("Чтото не так.");


            var values = Series.EMA(currData, currPeriod);
            return values.Last();
        }
    }
}