﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using TSLab.Script.Handlers;
using TSLab.Script.Helpers;

namespace RusAlgo.Handlers.Public.Adaptive
{
    [HandlerCategory("RusAlgo")]
    [HandlerName("Adaptive MACD")]
    [InputInfo(0, "Данные")]
    [InputInfo(1, "Период быстрой")]
    [InputInfo(1, "Период медленной")]
    public class AMacd : IThreeSourcesHandler, IDoubleInputs, IDoubleReturns, IStreamHandler, IValuesHandlerWithNumber, IContextUses
    {
        private double[] _dataCache;

        public IContext Context { set; private get; }

        public IList<double> Execute(IList<double> price, IList<double> fastPeriod, IList<double> slowPeriod)
        {
            var values = new double[Context.BarsCount];

            for (var i = 0; i < Context.BarsCount; i++)
            {
                var currFastPeriod = Math.Max(Convert.ToInt32(fastPeriod[i]), 1);
                var currSlowPeriod = Math.Max(Convert.ToInt32(slowPeriod[i]), 1);

                var fastValue = Context.GetData("AdaptiveMacdRusAlgo",
                                                new[]
                                                    {
                                                        currFastPeriod.ToString(CultureInfo.InvariantCulture),
                                                        price.GetHashCode().ToString(CultureInfo.InvariantCulture)
                                                    },
                                                () => Series.EMA(price, currFastPeriod));

                var slowValue = Context.GetData("AdaptiveMacdRusAlgo",
                                                new[]
                                                    {
                                                        currSlowPeriod.ToString(CultureInfo.InvariantCulture),
                                                        price.GetHashCode().ToString(CultureInfo.InvariantCulture)
                                                    },
                                                () => Series.EMA(price, currSlowPeriod));
                
                values[i] = fastValue[i] - slowValue[i];
            }

            return values;
        }

        public double Execute(double price, double fastPeriod, double slowPeriod, int barNum)
        {
            if (_dataCache == null)
                _dataCache = new double[Context.BarsCount];

            _dataCache[barNum] = price;

            var currFastPeriod = Math.Max(Convert.ToInt32(fastPeriod), 1);
            var currSlowPeriod = Math.Max(Convert.ToInt32(slowPeriod), 1);
            var maxPeriod = Math.Max(currFastPeriod, currSlowPeriod);

            // собираем все данные начиная с текущей свечки и назад на величину периода
            // один раз данные выбираем по большему периоду, чтобы не копировать два раза
            // всякие индюки на базе EMA имеют трабл с бесконечной памятью. Даем больше данных чтобы сгладить эту шнягу.
            var currData = _dataCache.CopyCountTo(barNum, maxPeriod*5 , false);

            // проверочка на случай кривизны
            if (currData.Last() != price)
                throw new Exception("Чтото не так.");


            var fastValue = Series.EMA(currData, currFastPeriod);
            var slowValue = Series.EMA(currData, currSlowPeriod);

            return fastValue.Last() - slowValue.Last();
        }
    }
}