﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using TSLab.Script;
using TSLab.Script.Handlers;
using TSLab.Script.Helpers;

namespace RusAlgo.Handlers.Public.Adaptive
{
    [HandlerCategory("RusAlgo")]
    [HandlerName("Adaptive StochK")]
    [InputInfo(0, "Инструмент")]
    [InputInfo(1, "Период")]
    public class aStochK : ITwoSourcesHandler, ISecurityInput0, IDoubleInput1, IDoubleReturns, IStreamHandler, IValuesHandlerWithNumber, IContextUses
    {
        public IContext Context { set; private get; }

        public IList<double> Execute(ISecurity sec, IList<double> period)
        {
            var values = new double[Context.BarsCount];

            for (var i = 0; i < Context.BarsCount; i++)
            {
                var currPeriod = Math.Max(Convert.ToInt32(period[i]), 1);
                var value = Context.GetData("AdaptiveStochKRusAlgo",
                                            new[]
                                                {
                                                    currPeriod.ToString(CultureInfo.InvariantCulture),
                                                    sec.GetHashCode().ToString(CultureInfo.InvariantCulture)
                                                }, () =>
                                                    {
                                                        var hnd = new StochK() {Context = Context, Period = currPeriod};
                                                        return hnd.Execute(sec);
                                                    });
                values[i] = value[i];
            }

            return values;
        }

        public double Execute(ISecurity sec, double period, int barNum)
        {
            var currPeriod = Math.Max(Convert.ToInt32(period), 1);

            var values =  Context.GetData("AdaptiveStochKRusAlgo",
                                            new[]
                                                {
                                                    currPeriod.ToString(CultureInfo.InvariantCulture),
                                                    sec.GetHashCode().ToString(CultureInfo.InvariantCulture)
                                                }, () =>
                                                {
                                                    var hnd = new StochK() { Context = Context, Period = currPeriod };
                                                    return hnd.Execute(sec);
                                                });
            
            return values[barNum];
        }
    }
}